c --- Individual modules for SURFGEN:
c      module const
c      module counts
c      module data
c      module isdcodes
c      module userinf
c

c-----------------------------------------------------------------------
      module const
c-----------------------------------------------------------------------
c --- SURFGEN   Version: 7.0.0          Level: 150330           /CONST/
c               D. Strimaitis, Exponent Inc.
c
c --- Assigned Constants
c-----------------------------------------------------------------------

      character(len=12) :: cversion,clevel
      integer           :: imiss4, izero
      real              :: rmiss4, rzero

c --- Assignments

      data cversion/'7.0.0       '/
      data clevel/'150330      '/

      data imiss4/9999/, rmiss4/9999.0/
      data izero/0/, rzero/0.0/

c --- Variables:
c ---------------
c         CVERSION - char*12    - Version of SURFGEN
c           CLEVEL - char*12    - Level of SURFGEN
c           IMISS4 - integer    - Missing value indicator for output
c           RMISS4 - real       - Missing value indicator for output
c            IZERO - integer    - Integer zero
c            RZERO - real       - Real zero
c-----------------------------------------------------------------------
      end module const
c-----------------------------------------------------------------------


c-----------------------------------------------------------------------
      module counts
c-----------------------------------------------------------------------
c --- SURFGEN   Version: 7.0.0          Level: 150330          /COUNTS/
c               D. Strimaitis, Exponent Inc.
c
c --- Counts for reporting missings/calms
c-----------------------------------------------------------------------
      parameter(nvar=8)

      character(len=20)     :: cvar(nvar)
      integer               :: npdproc
      integer, allocatable  :: nmiss1(:,:)                              ! (nvar,nfiles)
      integer, allocatable  :: nmiss(:,:)                               ! (nvar,nfiles+1)
      integer, allocatable  :: ncalm(:)                                 ! (nfiles)

c --- Assignments
      data cvar/'wind speed          ','wind direction      ',
     &          'temperature         ','relative humidity   ',
     &          'station pressure    ','ceiling height      ',
     &          'cloud cover         ','precipitation code  '/

c --- Variables:
c ---------------
c          NPDPROC - integer    - Number of output periods processed
c             NVAR - integer    - Number of output variables
c           CVAR() - char*20    - Names of output variables
c         NMISS1() - integer    - Local counter for number missings
c                                 for each of the variables at each
c                                 station for the current output period
c                                 (1,_) wind speed
c                                 (2,_) wind direction
c                                 (3,_) temperature
c                                 (4,_) relative humidity
c                                 (5,_) pressure
c                                 (6,_) ceiling height
c                                 (7,_) cloud cover
c                                 (8,_) precipitation code
c          NMISS() - integer    - Number of each variable missing at
c                                 each stn over all output periods
c                                 NFILES+1 = no. missing at All Stations
c                                 (1,_) wind speed
c                                 (2,_) wind direction
c                                 (3,_) temperature
c                                 (4,_) relative humidity
c                                 (5,_) pressure
c                                 (6,_) ceiling height
c                                 (7,_) cloud cover
c                                 (8,_) precipitation code
c          NCALM() - integer    - Number calm wind periods each stn
c-----------------------------------------------------------------------
      end module counts
c-----------------------------------------------------------------------


c-----------------------------------------------------------------------
      module data
c-----------------------------------------------------------------------
c --- SURFGEN   Version: 7.0.0          Level: 150330            /DATA/
c               D. Strimaitis, Exponent Inc.
c
c --- SURF.DAT and PRECIP.DAT Data Store
c-----------------------------------------------------------------------
c --- Data are stored for current period and 'next' period to identify
c --- proper output data record
c-----------------------------------------------------------------------

      integer*8             :: n8datim1,n8datim2
      integer               :: nyr1,njul1,nhr1,nsec1
      integer               :: nyr2,njul2,nhr2,nsec2

      integer,  allocatable :: idone(:),ispecial(:)                     ! (nfiles)
      integer*8,allocatable :: i8datim1(:,:),i8datim2(:,:)              ! (2,nfiles)
      integer,  allocatable :: iceil(:,:),irh(:,:),ipcode(:,:)          ! (3,nfiles)
      integer,  allocatable :: icco(:,:),icct(:,:),iccs(:,:)            ! (3,nfiles)
      real,     allocatable :: ws(:,:),wd(:,:),tk(:,:),pres(:,:)        ! (3,nfiles)
      real,     allocatable :: prate(:,:)                               ! (3,nfiles)

c --- Variables:
c ---------------
c --- Control
c          IDONE() - integer    - Status flag (0=not finished)
c       ISPECIAL() - integer    - Special observation flag for
c                                 new observation (just read)
c                                  0=regular, 1=special
c
c --- Date-time of output period
c         N8DATIM1 - integer*8  - Date-Time (YYYYJJJHHSSSS) at pd start
c       NYR1,NJUL1 - integer    - Date (YYYY, JJJ) at pd start
c       NHR1,NSEC1 - integer    - Time (HH, SSSS) at pd start
c         N8DATIM2 - integer*8  - Date-Time (YYYYJJJHHSSSS) at pd end
c       NYR2,NJUL2 - integer    - Date (YYYY, JJJ) at pd end
c       NHR2,NSEC2 - integer    - Time (HH, SSSS) at pd end
c
c --- Date-time of observation
c       (1,nfiles) = Current observation within current period
c       (2,nfiles) = New observation (just read)
c       I8DATIM1() - integer*8  - Date-Time (YYYYJJJHHSSSS) at start
c       I8DATIM2() - integer*8  - Date-Time (YYYYJJJHHSSSS) at end
c
c --- Observation Variables
c       (1,nfiles) = Current REGULAR observation
c       (2,nfiles) = New observation (either regular or special)
c       (3,nfiles) = Current SPECIAL observation
c             WS() - real       - Wind speed (m/s)
c             WD() - real       - Wind direction (degrees)
c          ICEIL() - integer    - Ceiling height (100s of ft)
c           ICCO() - integer    - Opaque sky cover (tenths)
c           ICCT() - integer    - Total sky cover (tenths)
c           ICCS() - integer    - Sky cover summation (tenths)
c             TK() - real       - Temperature (deg. K)
c            IRH() - integer    - Relative humidity (%)
c           PRES() - real       - Surface pressure (mb)
c         IPCODE() - integer    - Precipitation code
c          PRATE() - real       - Precip. rate  (mm/hr) 
c
c-----------------------------------------------------------------------
      end module data
c-----------------------------------------------------------------------


c-----------------------------------------------------------------------
      module head
c-----------------------------------------------------------------------
c --- SURFGEN   Version: 7.0.0          Level: 150330            /HEAD/
c               D. Strimaitis, Exponent Inc.
c
c --- Selected header information for output files
c-----------------------------------------------------------------------

      character(len=16) :: surfset,surfver,precset,precver
      character(len=64) :: datadoc
      integer           :: ncomment

c --- Assignments
      data surfset/'SURF.DAT        '/, surfver/'2.1             '/
      data precset/'PRECIP.DAT      '/, precver/'2.1             '/
      data datadoc/'Start and End Times with Seconds'/

c --- Variables:
c ---------------
c          SURFSET - char*16    - SURF.DAT dataset name
c          SURFVER - char*16    - SURF.DAT dataset version
c          PRECSET - char*16    - PRECIP.DAT dataset name
c          PRECVER - char*16    - PRECIP.DAT dataset version
c          DATADOC - char*64    - Additional dataset documentation
c         NCOMMENT - integer    - Number of comment lines in header
c-----------------------------------------------------------------------
      end module head
c-----------------------------------------------------------------------



c-----------------------------------------------------------------------
      module isdcodes
c-----------------------------------------------------------------------
c --- SURFGEN   Version: 7.0.0          Level: 150330        /ISDCODES/
c               D. Strimaitis, Exponent Inc.
c
c --- ISD (Integrated Surface Data) Codes
c-----------------------------------------------------------------------

      integer               :: maxrecl_isd, nctrl_isd, nmand_isd
      integer               :: itmiss_isd, iwsmiss_isd, iwdmiss_isd
      integer               :: islpmiss_isd, iceilmiss_isd
      integer               :: itfac_isd, iwsfac_isd, iwdfac_isd
      integer               :: islpfac_isd, iceilfac_isd
      integer               :: nadd_isd
      character(len=20)     :: cqfmand_isd,cfail_isd
      integer, allocatable  :: length_add(:), iuse_add(:)               ! (nadd_isd)

      character(len=3), allocatable  :: code_add(:)                     ! (nadd_isd)
      character(len=80), allocatable :: cdoc_add(:)                     ! (nadd_isd)

c --- Assignments
c ---------------
      data itmiss_isd/9999/, iwsmiss_isd/9999/, iwdmiss_isd/999/
      data islpmiss_isd/99999/, iceilmiss_isd/99999/
      data itfac_isd/10/, iwsfac_isd/10/, iwdfac_isd/1/
      data islpfac_isd/10/, iceilfac_isd/1/

c --- Variables:
c ---------------
c      MAXRECL_ISD - integer  - Max number characters in ISD record
c        NCTRL_ISD - integer  - Number characters in CONTROL section
c        NMAND_ISD - integer  - Number characters in MANDATORY section
c       IWSFAC_ISD - integer  - Multiplied factor wind speed
c       IWDFAC_ISD - integer  - Multiplied factor wind direction
c        ITFAC_ISD - integer  - Multiplied factor temperatures
c      ISLPFAC_ISD - integer  - Multiplied factor pressure
c     ICEILFAC_ISD - integer  - Multiplied factor ceiling ht
c      IWSMISS_ISD - integer  - Missing value wind speed
c      IWDMISS_ISD - integer  - Missing value wind direction
c       ITMISS_ISD - integer  - Missing value temperatures
c     ISLPMISS_ISD - integer  - Missing value pressure
c    ICEILMISS_ISD - integer  - Missing value ceiling ht
c      CQFMAND_ISD - char*20  - All single-character quality flags for
c                               MANDATORY section
c        CFAIL_ISD - char*20  - Single-character quality flags for
c                               invalid data in MANDATORY section
c         NADD_ISD - integer  - Number of ADDitional elements on record
c     LENGTH_ADD() - integer  - Number of characters in each element
c       IUSE_ADD() - integer  - Action for each element
c                               0 = skip data
c                               1 = read data
c       CODE_ADD() - char*3   - Code name for each element
c       CDOC_ADD() - char*80  - Documentation string for each element
c-----------------------------------------------------------------------
      end module isdcodes
c-----------------------------------------------------------------------


c-----------------------------------------------------------------------
      module userinf
c-----------------------------------------------------------------------
c --- SURFGEN   Version: 7.0.0          Level: 150330         /USERINF/
c               D. Strimaitis, Exponent Inc.
c
c --- Control file information
c-----------------------------------------------------------------------

      logical        :: lcfiles, lsurf, lprecip, lpmiss

      integer        :: nfiles, itimestep, nheadr, idate_col, itime_col
      integer        :: iws_col, iwd_col, it_col, irh_col, icldht_col
      integer        :: iceil_col, iccvr_col, ip_col, ipr_col
      integer        :: kyrb,kmob,kdyb,julb,khrb,ksecb
      integer        :: kyre,kmoe,kdye,jule,khre,ksece
      integer*8      :: k8bdatim,k8edatim
      integer, allocatable             :: isectzadj(:),isec2utc(:)      ! (nfiles)

      character(len=256)               :: clstfile, csrffile, cprcfile
      character(len=256)               :: cisd_code
      character(len=256), allocatable  :: cmetfile(:)                   ! (nfiles)
      character(len=256), allocatable  :: cmet6405(:),cmet6406(:)       ! (nfiles)
      character(len=20),  allocatable  :: cstaid(:)                     ! (nfiles)
      character(len=8),   allocatable  :: citzone(:),ctype(:)           ! (nfiles)
      character(len=8),   allocatable  :: cwsadj(:)                     ! (nfiles)
      character(len=10)                :: cs_date,ce_date
      character(len=8)                 :: cs_time, ce_time, cotzone
      character(len=10)                :: ctselect, cdate_fmt
      character(len=10)                :: cws_unit, cwd_unit, ct_unit
      character(len=10)                :: crh_unit, cceil_unit
      character(len=10)                :: cccvr_unit, cp_unit, cpr_unit

c --- Assignments
c ---------------
      data lcfiles/.FALSE./, lsurf/.TRUE./
      data lprecip/.FALSE./, lpmiss/.TRUE./
      data clstfile/'surfgen.lst'/
      data csrffile/'surf.dat'/, cprcfile/'precip.dat'/
      data cisd_code/'isd_codes.dat'/
      data ctselect/'LAST'/
c --- Initial values of date-time variables allow either format to be
c --- processed in the control file - code was developed expecting the
c --- string format (YYYY-MM-DD and HH:MM:SS), but may now use the 
c --- individual variables
      data cs_date/'9999-99-99'/,ce_date/'9999-99-99'/
      data cs_time/'99:99:99'/,ce_time/'99:99:99'/


c --- Variables:
c ---------------
c           NFILES - integer  - Number of input met data files
c        ITIMESTEP - integer  - Output file timestep (seconds)
c          CS_DATE - char*10  - Beginning date of run (YYYY-MM-DD)
c          CS_TIME - char*8   - Beginning time of run (HH:MM:SS)
c          CE_DATE - char*10  - Ending date of run (YYYY-MM-DD)
c          CE_TIME - char*8   - Ending time of run (HH:MM:SS)
c        KYRB,KYRE - integer  - Begin & End year of run (YYYY)
c        KMOB,KMOE - integer  - Begin & End month of run (MM)
c        KDYB,KDYE - integer  - Begin & End day of run (DD)
c        JULB,JULE - integer  - Begin & End Julian day of run (JJJ)
c        KHRB,KHRE - integer  - Begin & End hour of run (HH) [time]
c      KSECB,KSECE - integer  - Begin & End second of run (SSSS)
c         K8BDATIM - integer*8- Begin date-time of run (YYYYJJJHHSSSS)
c         K8EDATIM - integer*8- End date-time of run (YYYYJJJHHSSSS)
c
c
c         CLSTFILE - char*256 - Path & filename for output list file
c                               (default: SURFGEN.LST)
c         CSRFFILE - char*256 - Path & filename for output SURF.DAT
c                               (default: SURF.DAT)
c         CPRCFILE - char*256 - Path & filename for output PRECIP.DAT
c                               (default: PRECIP.DAT)
c        CISD_CODE - char*256 - Path & filename for input ISD code
c                               definition file (ADD group of variables)
c                               (default: ISD_CODES.DAT)
c       CMETFILE() - char*256 - Path & filename for each met data file
c                               (no default)
c       CMET6405() - char*256 - Path & filename for each ASOS 6405 file
c                               (no default)
c       CMET6406() - char*256 - Path & filename for each ASOS 6406 file
c                               (no default)
c         CSTAID() - char*20  - Station ID for each met data file
c         CWSADJ() - char*8   - Wind speed adjustment option
c                               NONE    :  no adjustment
c                               ASOSBIAS: Add 0.5 knot
c                               This is used to remove a half-knot
c                               bias present in ASOS reports that
c                               arises from truncating rather than
c                               rounding wind speed to integer knots
c                               before being written to the file
c          CTYPE() - char*8   - Type of each met data file
c                               ISD  : (Integrated Surface Data) files
c                                      with hourly data
c                               ASOS1: station files with subhourly data
c                               CSV  : (comma-separated variable) files
c                                      with subhourly data
c        CITZONE() - char*8   - Time zone for each met data file
c          COTZONE - char*8   - Time zone for output data files
c      ISECTZADJ() - integer  - Time zone adjustment:  SECONDS to ADD
c                               to file times to get output time zone
c       ISEC2UTC() - integer  - SECONDS to ADD to file times to get UTC
c
c          LCFILES - logical  - Switch indicating if all characters in
c                               the filenames are to be converted to
c                               lower case letters (LCFILES=T) or
c                               converted to UPPER case letters
c                               (LCFILES=F).
c           LSURF - logical  -  Create SURF.DAT file?
c         LPRECIP - logical  -  Create PRECIP.DAT file?
c          LPMISS - logical  -  Switch indicating if precipitation
c                               is set to zero when missing on a valid
c                               record
c                               T:  IPCODE & Precip Rate = 0
c                               F:  IPCODE & Precip Rate = 9999
c
c --- Configuration of CSV file
c           NHEADR - integer  - Number of header records in CSV file
c        IDATE_COL - integer  - Column for date in CSV record
c        ITIME_COL - integer  - Column for time in CSV record
c          IWS_COL - integer  - Column for wind speed in CSV record
c          IWD_COL - integer  - Column for wind direction in CSV record
c           IT_COL - integer  - Column for temperature in CSV record
c           IP_COL - integer  - Column for sfc. pressure in CSV record
c          IPR_COL - integer  - Column for precip rate in CSV record
c          IRH_COL - integer  - Column for rel. humidity in CSV record
c       ICLDHT_COL - integer  - Column for cloud desc. in CSV record
c        ICEIL_COL - integer  - Column for ceiling ht in CSV record
c        ICCVR_COL - integer  - Column for cloud cover in CSV record
c         CTSELECT - char*10  - Time-pd selection EXACT or NEAREST
c        CDATE_FMT - char*10  - Format for date in CSV record
c         CWS_UNIT - char*10  - Units for wind speed in CSV record
c         CWD_UNIT - char*10  - Units for wind direction in CSV record
c          CT_UNIT - char*10  - Units for temperature in CSV record
c          CP_UNIT - char*10  - Units for sfc. pressure in CSV record
c         CPR_UNIT - char*10  - Units for precip rate in CSV record
c         CRH_UNIT - char*10  - Units for rel. humidity in CSV record
c       CCEIL_UNIT - char*10  - Units for ceiling in CSV record
c      CCCVRT_UNIT - char*10  - Units for cloud cover in CSV record
c-----------------------------------------------------------------------
      end module userinf
c-----------------------------------------------------------------------


