c----------------------------------------------------------------------
c --- UTILS -- UTILITY SUBROUTINES
c----------------------------------------------------------------------
c
c --- UTILS      Version: 2.2.0    Level: 141010
c
c --- UPDATES:
c --- v2.1.1 (140801) to v2.2.0 (141010)
c        1. Add error-report for file-open
c           NEW: OPEN_ERR
c
c --- v2.1 (140422) to v2.1.1 (140801)
c        1. Switch version notation to vn.n.n
c        2. Set 'cfrom' string in WEEK_DAY (not passed in arg list)
c           MODIFIED: WEEK_DAY
c
c --- v2.0 (140219) to v2.1 (140422)
c        1. Add UNIX-style command-line-argument call
c           MODIFIED: GETCMD
c
c --- v1.23 (140106) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.  This removes the many checks on IERR from
c           calling subroutines.
c
c           *** This change is NOT COMPATIBLE with existing codes
c           *** because of the change in the argument list!
c
c           MODIFIED: CMPRSIO, UTC_ZONE, MK_CASE, DATE_HRSEC, DATE_HR
c                     CAL_DAY, WEEK_DAY, TIME_INCR, TIME_DIFF
c                     COND_HR_DAY, GET_RLL
c        2. Add sub to characterize relationship between 2 time periods
c           NEW: TIMECHK
c        3. Add sub to convert date strings using month/day
c           NEW: DATEMD_HR
c --- v1.22 (131218) to v1.23 (140106)
c        1. Add sub to obtain day-of-week
c           NEW: WEEK_DAY
c        2. Add screen to GET_RLL to demand inputs be in the following
c           ranges:
c              0E to 180E
c              0W to 180W
c              0N to 90N
c              0S to 90S
c --- v1.21 (130605) to v1.22 (131218)
c        1. Revert to implicit INT_8 to INT_4 conversions
c           MODIFIED: DATE_HRSEC, TIME_DIFF
c        2. Allow seconds from 0-3600 in date-times (had been 0-3599)
c           MODIFIED: DATE_HRSEC
c        3. Do not halt if control file assignments span multiple
c           lines, or if they have multiple assignments on 1 line.
c           These routines are a subset of those needed to process
c           complex control files.
c           MODIFIED: DELIM
c        4. Add sub to condition date-time at hour and day boundaries
c           Operates on YYYYJJJHHSSSS integer
c           NEW     : COND_HR_DAY          
c --- v1.2 (130419) to v1.21 (130605)
c        1. Revise local integer names to start with implicit range
c           of characters: i,j,k,l,m,n
c           and revise local character names to start with: c or a
c           MODIFIED: MK_CASE, DELIM, DATE_HRSEC, DATE_HR, CAL_DAY
c                     TIME_INCR, TIME_DIFF
c --- v1.1 (120915) to v1.2 (130419)
c        1. Add subr to read Lat/Lon string into real Lat or Lon
c           NEW     : GET_RLL
c --- v1.0 (120518) to v1.1 (120915)
c        1. Change file pathname variables from C*132 to C*256
c           MODIFIED: GETCMD
c
c----------------------------------------------------------------------
c --- System Interface
c      subroutine getcmd
c      subroutine open_err
c --- Data I/O
c      subroutine cmprsio
c      subroutine wrbdat
c      subroutine rdbdat
c --- String Manipulations (Control File Inputs)
c      subroutine utc_zone
c      subroutine mk_case
c      subroutine rm_blank
c      subroutine delim
c --- Date/Time Manipulations
c      subroutine date_hrsec
c      subroutine date_hr
c      subroutine cal_day
c      subroutine week_day
c      subroutine time_incr
c      subroutine time_diff
c      subroutine cond_hr_day
c      subroutine timechk
c --- Other
c      subroutine get_rll

c----------------------------------------------------------------------
      subroutine getcmd(arg1)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 140422           GETCMD
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Get first command line argument
c
c               This is system and compiler-dependent, configured
c               here for the Lahey compiler for the Windows operating
c               system.  Modify this subroutine to configure for
c               other compilations.  For example, the Lahey GETCL
c               can be replaced with the GETARG call used by other
c               systems, with suitable declarations.  Consult your
c               FORTRAN users guide for details.
c
c --- UPDATES:
c --- v1.1 (120915) to v2.1 (140422)
c        1. Add UNIX-style calls as in PGI FORTRAN
c --- v1.0 (120518) to v1.1 (120915)
c        1. Change default pathname variable from C*132 to C*256
c
c --- INPUTS:
c           ARG1 - character* - Default command line argument #1
c
c --- OUTPUTS:
c           ARG1 - character* - Returned command line argument #1
c                               (updated if argument provided)
c
c --- GETCMD called by:  ()
c --- GETCMD calls:      GETCL - Lahey system routine
c                        GETARG - PGI (VAX/VMS) system routine
c                        IARGC - PGI (VAX/VMS) system routine
c----------------------------------------------------------------------
      implicit none

      integer            :: iargc
      character(len=*)   :: arg1
      character(len=256) :: cdeflt

c --- Assign default
      cdeflt=arg1

cc --- Lahey version of system call
cc --- ----------------------------
cc --- Read current argument (if any) from the command line
c      call GETCL(arg1)
cc --- Pass default when no command line arguments are given
c      if(arg1(1:1) .EQ. ' ') arg1=cdeflt

c --- PGI version of system call
c --- --------------------------
c --- Read current argument (if any) from the command line
      if(IARGC().GT.0) then
         call GETARG(1,arg1)
      else
         arg1=cdeflt
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine open_err(iolst,cfrom,cftype,cfname,iunit)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 141010         OPEN_ERR
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Report error in opening a file
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c                                   (<0 if not available)
c               CFROM - char*     - Called-From string to report error
c              CFTYPE - char*     - File-type string
c              CFNAME - char*     - File-name string
c               IUNIT - integer   - File unit number
c
c --- OUTPUT:
c
c --- OPEN_ERR called by:  ()
c --- OPEN_ERR calls:      
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: cfrom,cftype,cfname
      integer          :: iolst, iunit

      if(iolst.GT.0) then
         write(iolst,*)
         write(iolst,*)'ERROR opening '//TRIM(cftype)
         write(iolst,*)'   File Name: '//TRIM(cfname)
         write(iolst,*)'   File Unit: ',iunit
         write(iolst,*)'Problem reported from '//TRIM(cfrom)
         write(iolst,*)
         write(iolst,*)'The file may not exist in this location'
         write(iolst,*)'Check the spelling of the name and the location'
         write(*,*)
         stop 'ERROR: File not found -- see list file'
      else
         write(*,*)
         write(*,*)'ERROR opening '//TRIM(cftype)
         write(*,*)'   File Name: '//TRIM(cfname)
         write(*,*)'   File Unit: ',iunit
         write(*,*)'Problem reported from '//TRIM(cfrom)
         write(*,*)
         write(*,*)'The file may not exist in this location'
         write(*,*)'Check the spelling of the name, and the location'
         stop
      endif

      end

c----------------------------------------------------------------------
      subroutine cmprsio(iolst,iodat,cfrom,action,xdata,ndata,xcmprs,
     &                   ncmprs,clabel)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 140219          CMPRSIO
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read & expand a compressed data record from binary file
c               or compress & write data record to binary file
c               (Compression replaces consecutive zeros with a single
c               negative count of the number of zeros)
c
c --- UPDATES:
c --- v1.0 (120518) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c               IODAT - integer   - Unit number of binary data file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*4    - Action to perform (GETR or PUTR)
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT: (depends on ACTION)
c        XDATA(ndata) - real arr. - Array of uncompressed data
c               NDATA - integer   - Number of values in full array
c      XCMPRS(ncmprs) - real arr. - Array compressed data
c              NCMPRS - integer   - Number of words in compressed
c                                   data record
c              CLABEL - char*     - Character record ID
c
c --- CMPRSIO called by: ()
c --- CMPRSIO calls:     WRBDAT
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: clabel, cfrom
      character*4 :: action
      real :: xcmprs(ncmprs),xdata(ndata)
      integer :: iolst,iodat,ndata,ncmprs

c --- Locals
      integer :: i,j,nxpnd,nzero

      if(action.EQ.'GETR') then
c ---    Initialization
         nxpnd=0

c ---    Read a compressed data record
         read(iodat) clabel,xcmprs

c ---    Expand the data to physical representation
         do i=1,ncmprs
            if(xcmprs(i).GT.0.0)then
c ---          Store value
               nxpnd=nxpnd+1
               xdata(nxpnd)=xcmprs(i)
            else
c ---          Store number of zeros indicated
               do j=1,NINT(-xcmprs(i))
                  nxpnd=nxpnd+1
                  xdata(nxpnd)=0.0
               enddo
            endif
         enddo

c ---    QA checks
         if(nxpnd.NE.ndata) then
            write(iolst,*)'ERROR in CMPRSIO -- Invalid expanded array '
            write(iolst,*)'   Number of words expected: ',ndata
            write(iolst,*)'   Number of words found   : ',nxpnd
            write(iolst,*)'Called from  '//cfrom
            stop 'HALT in CMPRSIO -- see list file'
         endif

      elseif(action.EQ.'PUTR') then
c ---    Initialization
         nzero=0
         ncmprs=0

c ---    Loop over all words in data array
         do i=1,ndata

c ---       QA:  Negative data values cannot be used here
            if(xdata(i).LT.0.0) then
               write(iolst,*)'ERROR in CMPRSIO -- Negative data '//
     &              'values cannot be used with COMPRESSION option'
               write(iolst,*)'Data ID, index, value = ',clabel,
     &                        i,xdata(i)
               write(iolst,*)'Called from  '//cfrom
               stop 'HALT in CMPRSIO -- see list file'
            endif

            if(xdata(i).EQ.0.0) then
c ---          Just increment zero counter for now
               nzero=nzero+1
            elseif(nzero.EQ.0) then
c ---          Non-zero data and no preceding zeros: Place into output
               ncmprs=ncmprs+1
               xcmprs(ncmprs)=xdata(i)
            else
c ---          Non-zero data with preceding zeros
c ---          Write and reset compression number
               ncmprs=ncmprs+1
               xcmprs(ncmprs)=-(FLOAT(nzero)+0.0001)
               nzero=0
c ---          Write current non-zero value
               ncmprs=ncmprs+1
               xcmprs(ncmprs)=xdata(i)
            endif
         enddo

c ---    Treat case of last array value = zero
         if(nzero.GT.0) then
            ncmprs=ncmprs+1
            xcmprs(ncmprs)=-(FLOAT(nzero)+0.0001)
         endif

c ---    Write header and compressed data records
         write(iodat) ncmprs
         call WRBDAT(iodat,clabel,xcmprs,ncmprs,1)

      else
         write(iolst,*)'ERROR in CMPRSIO -- Invalid ACTION '
         write(iolst,*)'   ACTION expected: GETR or PUTR'
         write(iolst,*)'   ACTION found   : ',action
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in CMPRSIO -- see list file'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine wrbdat(io,clabel,data,n,m)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 120518           WRBDAT
c               J. Scire, Exponent Inc.
c
c --- PURPOSE:  Write a gridded data record to binary file
c               (character identifier and a 2-D data array)
c
c --- INPUT:
c              IO - integer     - Unit no. of output file
c          clabel - character*  - Data label
c       DATA(n,m) - real arr.   - Data array
c             N,M - integer     - Array size
c
c --- OUTPUT:  none
c
c --- WRBDAT called by:  ()
c --- WRBDAT calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Calling routine is responsible for ID string length
      character(len=*) :: clabel
      real :: data(n,m)
      integer :: io,n,m

      write(io) clabel,data

      return
      end

c----------------------------------------------------------------------
      subroutine rdbdat(io,clabel,data,n,m)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 120518           RDBDAT
c               J. Scire, Exponent Inc.
c
c --- PURPOSE:  Read a gridded data record from binary file
c               (character identifier and a 2-D data array)
c
c --- INPUT:
c              IO - integer     - Unit no. of input file
c          clabel - character*  - Data label
c       DATA(n,m) - real arr.   - Data array
c             N,M - integer     - Array size
c
c --- OUTPUT:  none
c
c --- RDBDAT called by:  ()
c --- RDBDAT calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Calling routine is responsible for ID string length
      character(len=*) :: clabel
      real :: data(n,m)
      integer :: io,n,m

      read(io) clabel,data

      return
      end

c----------------------------------------------------------------------
      subroutine utc_zone(iolst,cfrom,action,char_tz,real_tz,intg_tz)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 140219         UTC_ZONE
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Converts UTC time zone designator between a character
c               string representation and either a real representation
c               or an integer representation
c
c --- UPDATES:
c --- v1.0 (120518) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*8    - Action to perform
c                                   (CHR2REAL or REAL2CHR)
c                                   (CHR2INTG or INTG2CHR)
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c             CHAR_TZ - char*8    - Time zone as character string that
c                                   indicates time to add to UTC to
c                                   obtain local time: UTC+HHMM
c             REAL_TZ - real      - Time zone as real that indicates
c                                   hours to ADD to local time to
c                                   obtain UTC time
c             INTG_TZ - integer   - Time zone as integer that indicates
c                                   hours to ADD to local time to
c                                   obtain UTC time
c
c --- UTC_ZONE called by:  ()
c --- UTC_ZONE calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      real :: real_tz
      integer :: intg_tz,iolst
      character(len=*) :: cfrom
      character(len=8) :: char_tz
      character(len=8) :: action

c --- Declare locals
      integer :: ihour,iminute

      if(action.EQ.'CHR2REAL') then
c ---    (a) Convert from character to real representation
         if(INDEX(char_tz,'0000').GT.0) then
            real_tz=0.0
            return
         endif
c ---    2-digit hour with sign, 2-digit minutes
         read(char_tz(4:6),'(i3)') ihour
         read(char_tz(7:8),'(i2)') iminute
c ---    Apply sign to minutes
         iminute=SIGN(iminute,ihour)
c ---    Form real as time to ADD to local time to obtain UTC
         real_tz=-(FLOAT(ihour)+FLOAT(iminute)/60.)

      elseif(action.EQ.'CHR2INTG') then
c ---    (b) Convert from character to integer representation
         if(INDEX(char_tz,'0000').GT.0) then
            intg_tz=0
            return
         endif
c ---    2-digit hour with sign, 2-digit minutes
         read(char_tz(4:6),'(i3)') ihour
         read(char_tz(7:8),'(i2)') iminute
c ---    QA:  minutes must be zero
         if(iminute.NE.0) then
            write(iolst,*)'ERROR in UTC_ZONE -- Invalid ACTION for zone'
            write(iolst,*)'   CHR2INTG requires zone with zero minutes'
            write(iolst,*)'   Zone found : ',char_tz
            write(iolst,*)'Called from  '//cfrom
            stop 'HALT in UTC_ZONE -- see list file'
         endif
c ---    Form integer as hours to ADD to local time to obtain UTC
         intg_tz=-ihour

      elseif(action.EQ.'REAL2CHR') then
c ---    (c) Convert from real to character representation
c ---    Create string template as time ADDED to UTC to obtain local
         if(real_tz.EQ.0.0) then
            char_tz='UTC+0000'
            return
         elseif(real_tz.LT.0.0) then
            char_tz='UTC+0000'
         else
            char_tz='UTC-0000'
         endif
c ---    Extract minutes as integer
         iminute=NINT(60.*MOD(ABS(real_tz),1.0))
c ---    Extract unsigned hours as integer
         ihour=INT(ABS(real_tz))
c ---    Place in string
         write(char_tz(5:6),'(i2.2)') ihour
         write(char_tz(7:8),'(i2.2)') iminute

      elseif(action.EQ.'INTG2CHR') then
c ---    (d) Convert from integer to character representation
c ---    Create string template as time ADDED to UTC to obtain local
         if(intg_tz.EQ.0) then
            char_tz='UTC+0000'
            return
         elseif(intg_tz.LT.0) then
            char_tz='UTC+0000'
         else
            char_tz='UTC-0000'
         endif
c ---    Place unsigned hours in string
         write(char_tz(5:6),'(i2.2)') IABS(intg_tz)

      else
         write(iolst,*)'ERROR in UTC_ZONE -- Invalid ACTION '
         write(iolst,*)'   ACTION expected: CHR2REAL or REAL2CHR'
         write(iolst,*)'   ACTION found   : ',action
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in UTC_ZONE -- see list file'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine mk_case(iolst,cfrom,action,cvar)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 140219          MK_CASE
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Imposes uniform case on a string
c
c --- UPDATES:
c --- v1.21 (130605) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.
c --- v1.0 (120518) to v1.21 (130605)
c        1. Revise local integer names to start with implicit range
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*     - Action to perform
c                                   (UPPER or LOWER)
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c                CVAR - char*     - String
c
c --- MK_CASE called by:  ()
c --- MK_CASE calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: cvar,action,cfrom
      integer :: iolst

c --- Declare locals
      integer :: iul,ilo(2),ihi(2),ishift(2)
      integer :: nc,ia,ic

c --- Define ASCII table pointers for conversion to UPPER and LOWER
c ---         (UPPER,LOWER)
      data ilo   / 97,65/
      data ihi   /122,90/
      data ishift/-32,32/

c --- Set conversion
      if(TRIM(action).EQ.'UPPER') then
         iul=1
      elseif(TRIM(action).EQ.'LOWER') then
         iul=2
      else
         write(iolst,*)'ERROR in MK_CASE -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: UPPER or LOWER'
         write(iolst,*)'   ACTION found   : ',action
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in MK_CASE -- see list file'
      endif

c --- String length
      nc=LEN(cvar)

c --- Process string
      do ic=1,nc
         ia=IACHAR(cvar(ic:ic))
         if(ia.GE.ilo(iul) .AND. ia.LE.ihi(iul)) then
            ia=ia+ishift(iul)
            cvar(ic:ic)=ACHAR(ia)
         endif
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine rm_blank(cvar)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 120518         RM_BLANK
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Moves all blanks to end of string
c
c --- INPUT:
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c                CVAR - char*     - String
c
c --- RM_BLANK called by:  ()
c --- RM_BLANK calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: cvar

c --- Declare locals
      integer :: nc,ic,ix

c --- Intrinsic fcn clears leading blanks
      cvar=ADJUSTL(cvar)

c --- March along string, overwriting any blanks
      nc=LEN_TRIM(cvar)
      ix=1
      do ic=1,nc
         if(cvar(ic:ic).NE.' ') then
            cvar(ix:ix)=cvar(ic:ic)
            ix=ix+1
         endif
      enddo
c --- Clear to end of string if needed
      if(ix.LE.nc) then
         do ic=ix,nc
            cvar(ic:ic)=' '
         enddo
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine delim(cdlim,cvar,id1,id2,ieq,lprocess)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 131218            DELIM
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Determines if CVAR is a line containing a control-file
c               assignment between delimiters and if so, it returns
c               the position of the 2 delimiters and the equals sign.
c               Only the first assignment on a line is processed.
c
c --- UPDATES:
c --- v1.21 (130605) to v1.22 (131218)
c        1. If more than 2 delimiters are in string, accept just
c           the first 2 (do not halt)
c        2. If single delimiter is on the line, skip it (no halt)
c --- v1.0 (120518) to v1.21 (130605)
c        1. Revise delimiter character name to start with 'c'
c
c --- INPUT:
c               CDLIM - char*     - Delimiter (should be 1 character)
c                CVAR - char*     - String
c
c --- OUTPUT:
c                 ID1 - integer   - Position of 1st delimiter in CVAR
c                 ID2 - integer   - Position of next delimiter in CVAR
c                 IEQ - integer   - Position of '=' in CVAR
c            LPROCESS - logical   - True if a good assignment
c
c --- DELIM called by:  ()
c --- DELIM calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      logical          :: lprocess
      character(len=*) :: cdlim,cvar
      integer          :: id1,id2,ieq

c --- Locals
      integer          :: idx

      lprocess=.TRUE.
      ieq=0
      id1=0
      id2=0

c --- Find position of markers
      id1=INDEX(cvar,cdlim)
      if(id1.EQ.0) then
         lprocess=.FALSE.
         return
      endif
      id2=INDEX(cvar,cdlim,back=.TRUE.)
      if(id1.GT.0 .AND. id2.GT.id1) then
c ---    There are at least 2 delimiters
c ---    Check for another delimiter between these
c ---    and take the first if another is found
         idx =INDEX(cvar(id1+1:id2-1),cdlim)
         if(idx.GT.0) id2=id1+idx
c ---    Equals sign between these?
         ieq=INDEX(cvar(id1:id2),'=')
         if(ieq.GT.0) ieq=ieq+id1-1
      else
c ---    One delimiter found
         lprocess=.FALSE.
c         write(*,*)'ERROR in Control File'
c         write(*,*)'Line contains only 1 delimiter'
c         write(*,*)'Assignment between delimiters is expected'
c         write(*,*)TRIM(cvar)
c         stop 'HALTED in DELIM'
      endif

c --- Must have an equals sign to process
      if(ieq.EQ.0) then
         lprocess=.FALSE.
         return
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine date_hrsec(iolst,cfrom,action,iyear,juld,ihour,
     &                      isecond,i8datetime)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140219       DATE_HRSEC
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Converts between individual date and time variables
c               and single integer*8 YYYYJJJHHSSSS variable
c
c --- UPDATES:
c --- v1.22 (131218) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.
c --- v1.21 (130605) to v1.22 (131218)
c        1. Revert to implicit INT_8 to INT_4 conversion
c        2. Allow seconds=3600 (up from 3599)
c --- v1.0 (120518) to v1.21 (130605)
c        1. Revise integer names to start with: i,j,k,l,m,n
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*     - Action to perform
c                                   TOINDV returns components
c                                   TOSNGL returns single integer
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c               IYEAR - integer   - Year (YYYY)
c                JULD - integer   - Julian day (JJJ)
c               IHOUR - integer   - Hour (HH)
c             ISECOND - integer   - Second (SSSS)
c          I8DATETIME - integer*8 - Combined date & time (YYYYJJJHHSSSS)
c
c --- DATE_HRSEC called by:  ()
c --- DATE_HRSEC calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action, cfrom
      integer          :: iolst,iyear,juld,ihour,isecond
      integer(kind=8)  :: i8datetime

c --- Locals
      logical lbreakup
      integer          :: ierr
      integer(kind=8)  :: i8yr,i8jd,i8hr,i8sc

c --- Initializations
      ierr=0

c --- Set conversion
      if(action.EQ.'TOSNGL') then
         lbreakup=.FALSE.
c ---    QA inputs
         if(iyear.GT.9999 .OR. iyear.LT.1) ierr=1
         if(juld.GT.366 .OR. juld.LT.1) ierr=1
         if(ihour.GT.24 .OR. ihour.LT.0) ierr=1
         if(isecond.GT.3600 .OR. isecond.LT.0) ierr=1
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in DATE_HRSEC -- Invalid inputs'
            write(iolst,*)'      YEAR =',iyear
            write(iolst,*)'JULIAN DAY =',juld
            write(iolst,*)'      HOUR =',ihour
            write(iolst,*)'    SECOND =',isecond
            write(iolst,*)'Called from  '//cfrom
            stop 'HALT in DATE_HRSEC -- see list file'
         endif
      elseif(action.EQ.'TOINDV') then
         lbreakup=.TRUE.
      else
         write(iolst,*)'ERROR in DATE_HRSEC -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: TOINDV or TOSNGL'
         write(iolst,*)'   ACTION found   : ',action
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in DATE_HRSEC -- see list file'
      endif

      if(lbreakup) then
c ---    Break long integer into components
         i8yr=i8datetime/1000000000_8
         i8jd=i8datetime/1000000_8-i8yr*1000_8
         i8hr=i8datetime/10000_8-i8yr*100000_8-i8jd*100_8
         i8sc=i8datetime-i8yr*1000000000_8-i8jd*1000000_8-
     &                   i8hr*10000_8
         iyear=i8yr
         juld= i8jd
         ihour=i8hr
         isecond=i8sc
      else
c ---    Combine components into long integer
         i8datetime=iyear*1000000000_8+juld*1000000_8+
     &              ihour*10000_8+isecond
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine date_hr(iolst,cfrom,action,iyear,juld,ihour,
     &                   idatehour)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140219          DATE_HR
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Converts between individual date and time variables
c               and single integer YYYYJJJHH variable
c
c --- UPDATES:
c --- v1.21 (130605) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.
c --- v1.0 (120518) to v1.21 (130605)
c        1. Revise integer names to start with: i,j,k,l,m,n
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*     - Action to perform
c                                   TOINDV returns components
c                                   TOSNGL returns single integer
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c               IYEAR - integer   - Year (YYYY)
c                JULD - integer   - Julian day (JJJ)
c               IHOUR - integer   - Hour (HH)
c           IDATEHOUR - integer   - Combined date & time (YYYYJJJHH)
c
c --- DATE_HR called by:  ()
c --- DATE_HR calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action, cfrom
      integer          :: iolst,iyear,juld,ihour,idatehour

c --- Locals
      logical  :: lbreakup
      integer  :: ierr

c --- Initializations
      ierr=0

c --- Set conversion
      if(action.EQ.'TOSNGL') then
         lbreakup=.FALSE.
c ---    QA inputs
         if(iyear.GT.9999 .OR. iyear.LT.1) ierr=1
         if(juld.GT.366 .OR. juld.LT.1) ierr=1
         if(ihour.GT.24 .OR. ihour.LT.0) ierr=1
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in DATE_HR -- Invalid inputs'
            write(iolst,*)'      YEAR =',iyear
            write(iolst,*)'JULIAN DAY =',juld
            write(iolst,*)'      HOUR =',ihour
            write(iolst,*)'Called from  '//cfrom
            stop 'HALT in DATE_HR -- see list file'
         endif
      elseif(action.EQ.'TOINDV') then
         lbreakup=.TRUE.
      else
         write(iolst,*)'ERROR in DATE_HR -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: TOINDV or TOSNGL'
         write(iolst,*)'   ACTION found   : ',action
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in DATE_HR -- see list file'
      endif

      if(lbreakup) then
c ---    Break integer into components
         iyear=idatehour/100000
         juld=idatehour/100-iyear*1000
         ihour=idatehour-iyear*100000-juld*100
      else
c ---    Combine components into single integer
         idatehour=iyear*100000+juld*100+ihour
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine datemd_hr(iolst,cfrom,action,iyear,imo,iday,ihour,
     &                     idatehour)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140219        DATEMD_HR
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Converts between individual date and time variables
c               and single integer YYYYMMDDHH variable
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*     - Action to perform
c                                   TOINDV returns components
c                                   TOSNGL returns single integer
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c               IYEAR - integer   - Year (YYYY)
c                 IMO - integer   - Month (MM)
c                IDAY - integer   - Day (DD)
c               IHOUR - integer   - Hour (HH)
c           IDATEHOUR - integer   - Combined date & time (YYYYMMDDHH)
c
c --- DATEMD_HR called by:  ()
c --- DATEMD_HR calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action, cfrom
      integer          :: iolst,iyear,imo,iday,ihour,idatehour

c --- Locals
      logical  :: lbreakup
      integer  :: ierr

c --- Initializations
      ierr=0

c --- Set conversion
      if(action.EQ.'TOSNGL') then
         lbreakup=.FALSE.
c ---    QA inputs
         if(iyear.GT.9999 .OR. iyear.LT.1) ierr=1
         if(imo.GT.12 .OR. imo.LT.1) ierr=1
c ---    Weak test on valid day-of-month
         if(iday.GT.31 .OR. iday.LT.1) ierr=1
         if(ihour.GT.24 .OR. ihour.LT.0) ierr=1
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in DATEMD_HR -- Invalid inputs'
            write(iolst,*)'      YEAR =',iyear
            write(iolst,*)'     MONTH =',imo
            write(iolst,*)'       DAY =',iday
            write(iolst,*)'      HOUR =',ihour
            write(iolst,*)'Called from  '//cfrom
            stop 'HALT in DATEMD_HR -- see list file'
         endif
      elseif(action.EQ.'TOINDV') then
         lbreakup=.TRUE.
      else
         write(iolst,*)'ERROR in DATEMD_HR -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: TOINDV or TOSNGL'
         write(iolst,*)'   ACTION found   : ',action
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in DATEMD_HR -- see list file'
      endif

      if(lbreakup) then
c ---    Break integer into components
         iyear=idatehour/1000000
         imo=idatehour/10000-iyear*100
         iday=idatehour/100-iyear*10000-imo*100
         ihour=idatehour-iyear*1000000-imo*10000-iday*100
      else
c ---    Combine components into single integer
         idatehour=iyear*1000000+imo*10000+iday*100+ihour
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine cal_day(iolst,cfrom,action,iyear,month,iday,julday)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140219          CAL_DAY
c               J. Scire, Exponent Inc.
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Convert between Julian day and Gregorian date
c
c --- UPDATES:
c --- v1.21 (130605) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.
c --- v1.0 (120518) to v1.21 (130605)
c        1. Revise integer names to start with: i,j,k,l,m,n
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*     - Action to perform
c                                   TOJULIAN returns Julian day
c                                   TOGREGOR returns Gregorian date
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c               IYEAR - integer   - Year (YYYY)
c               MONTH - integer   - Month (MM)
c                IDAY - integer   - Day (DD)
c              JULDAY - integer   - Julian day (JJJ)
c
c --- CAL_DAY called by:  ()
c --- CAL_DAY calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action, cfrom
      integer          :: iolst,iyear,month,iday,julday

c --- Locals
      logical  :: ljulian
      integer  :: i,ndpmo(12,2),ncumday(12,2),leap,lastday(2),ierr

c --- Set number of days in each month (regular,leap)
      data ndpmo/31,28,31,30,31,30,31,31,30,31,30,31,
     &           31,29,31,30,31,30,31,31,30,31,30,31/

c --- Set number of days in year (regular,leap)
      data lastday/365,366/

c --- Set cumulative number of days to start of each
c --- month (regular,leap)
      data ncumday/0,31,59,90,120,151,181,212,243,273,304,334,
     &             0,31,60,91,121,152,182,213,244,274,305,335/

c --- Initializations
      ierr=0

c --- Set leap year index (1=regular, 2=leap)
      leap=1
      if(MOD(iyear,4).EQ.0) leap=2

c --- Set conversion
      if(action.EQ.'TOJULIAN') then
         ljulian=.TRUE.
c ---    QA inputs
         if(month.GT.12 .OR. month.LT.1) ierr=1
         if(ierr.EQ.0) then
            if(iday.LT.1 .OR. iday.GT.ndpmo(month,leap)) ierr=1
         endif
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in CAL_DAY -- Invalid inputs'
            write(iolst,*)'      YEAR =',iyear
            write(iolst,*)'     MONTH =',month
            write(iolst,*)'       DAY =',iday
            write(iolst,*)'Called from  '//cfrom
            stop 'HALT in CAL_DAY -- see list file'
         endif
      elseif(action.EQ.'TOGREGOR') then
         ljulian=.FALSE.
c ---    QA inputs
         if(julday.GT.lastday(leap) .OR. julday.LT.1) ierr=1
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in CAL_DAY -- Invalid inputs'
            write(iolst,*)'      YEAR =',iyear
            write(iolst,*)'JULIAN DAY =',julday
            write(iolst,*)'Called from  '//cfrom
            stop 'HALT in CAL_DAY -- see list file'
         endif
      else
         write(iolst,*)'ERROR in CAL_DAY -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: TOJULIAN or TOGREGOR'
         write(iolst,*)'   ACTION found   : ',action
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in CAL_DAY -- see list file'
      endif

      if(ljulian) then
c ---    Convert date to the Julian day
         julday=ncumday(month,leap)+iday
      else
c ---    Convert Julian day to the date
         if(julday.LE.ncumday(7,leap)) then
            do i=6,1,-1
               if(julday.GT.ncumday(i,leap)) then
                  month=i
                  iday=julday-ncumday(i,leap)
                  return
               endif
            enddo
         else
            do i=12,7,-1
               if(julday.GT.ncumday(i,leap)) then
                  month=i
                  iday=julday-ncumday(i,leap)
                  return
               endif
            enddo
         endif
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine week_day(iolst,action,iyear,month,iday,julday,
     &                    iwkday,cwkday)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140801         WEEK_DAY
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Return day of week from Zeller's Congruence
c
c --- UPDATES:
c --- v2.0 (140219) to v2.1.1 (140801)
c        1. Set 'cfrom' string (not passed in arg list)
c --- v1.23 (140106) to v2.0 (140219)
c        1. Add new string argument CFROM to calls in place of ierr
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c              ACTION - char*     - Action to perform (type of date)
c                                   JULIAN uses Julian day (day-of-year)
c                                   GREGOR uses Gregorian date
c               IYEAR - integer   - Year (YYYY)
c               MONTH - integer   - Month (MM)
c                IDAY - integer   - Day (DD)
c              JULDAY - integer   - Julian day (JJJ)
c
c --- OUTPUT:
c
c              IWKDAY - integer   - Day of week (1=Sunday, 7=Saturday)
c              CWKDAY - character - Day of week
c
c --- WEEK_DAY called by:  ()
c --- WEEK_DAY calls:      CAL_DAY
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action
      character(len=9) :: cwkday
      integer          :: iwkday
      integer          :: iolst,iyear,month,iday,julday

c --- Locals
      character(len=9) :: cweek(7)
      character(len=8) :: cfrom
      integer          :: i,j,k,jk,iq,m

      data cweek/'SUNDAY   ','MONDAY   ','TUESDAY  ','WEDNESDAY',
     &           'THURSDAY ','FRIDAY   ','SATURDAY '/
      data cfrom/'WEEK_DAY'/

      SELECT CASE (action)

         CASE ("JULIAN")
            call CAL_DAY(iolst,cfrom,'TOGREGOR',jk,m,iq,julday)

         CASE ("GREGOR")
            jk=iyear
            m=month
            iq=iday

         CASE DEFAULT
            write(iolst,*)'ERROR in WEEK_DAY -- Invalid ACTION'
            write(iolst,*)'  Expected: JULIAN or GREGOR'
            write(iolst,*)'     Found: '//action
            stop 'HALT in WEEK_DAY -- see list file'

      END SELECT

c --- Treat JAN and FEB as months 13, 14 of previous year
      if(m.LE.2) then
         jk=jk-1
         m=m+12
      endif

c --- Separate the year into century and year of century
      j=jk/100
      k=MOD(jk,100)

c --- Using integer arithmetic, Zeller's formula is
c --- i(0-6) = (iq + ((m+1)*26)/10 + k +k/4 + j/4 +5j), MOD 7
      i=iq + ((m+1)*26)/10 + k +k/4 + j/4 +5*j
      iwkday=MOD(i,7)

c --- Saturday is 0 or 7
      if(iwkday.EQ.0) iwkday=7

c --- Set character result
      cwkday=cweek(iwkday)

      return
      end

c----------------------------------------------------------------------
      subroutine time_incr(iolst,cfrom,iyear,julday,ihour,isecond,
     &                     ndthr,ndtsec)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140219        TIME_INCR
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Add time interval (pos or neg) to a date-time
c               Date (Julian day) is updated accordingly
c
c --- UPDATES:
c --- v1.21 (130605) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.
c --- v1.0 (120518) to v1.21 (130605)
c        1. Revise integer names to start with: i,j,k,l,m,n
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c               NDTHR - integer   - Hours to ADD to time (may be <0)
c              NDTSEC - integer   - Seconds to ADD to time (may be <0)
c
c --- OUTPUT:
c                IERR - integer   - Error flag (0 = No Error)
c
c --- INPUT/OUTPUT:
c               IYEAR - integer   - Year (YYYY)
c              JULDAY - integer   - Julian day (JJJ)
c               IHOUR - integer   - Hour (HH)
c             ISECOND - integer   - Second (SSSS)
c
c --- TIME_INCR called by:  ()
c --- TIME_INCR calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: cfrom
      integer          :: iolst,iyear,julday,ihour,isecond
      integer          :: ndthr,ndtsec

c --- Locals
      integer          :: ndh,nds,ndtday,lastday

c --- Initializations
      ndtday=0

c --- Incremental hours and seconds must have the same sign
c --- (product must be zero or positive)
      if((ndthr*ndtsec) .LT. 0) then
         write(iolst,*)'ERROR in TIME_INCR -- Invalid increment'
         write(iolst,*)'Hours and seconds must have same sign'
         write(iolst,*)'NDTHR, NDTSEC =',ndthr,ndtsec
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in TIME_INCR -- see list file'
      endif

c --- Condition increment so that there are fewer than 3600 seconds
      ndh=ndthr
      nds=ndtsec
      if(ABS(nds).GT.3599) then
         ndh=ndh+nds/3600
         nds=MOD(nds,3600)
      endif

c --- Apply change in seconds
      isecond=isecond+nds
      if(isecond.LT.0) then
c ---    Move hours back 1 hour and seconds forward 3600
         ndh=ndh-1
         isecond=isecond+3600
      elseif(isecond.GE.3600) then
c ---    Move hours forward 1 hour and seconds back 3600
         ndh=ndh+1
         isecond=isecond-3600
      endif

c --- Condition increment so that there are fewer than 24 hours
      if(ABS(ndh).GT.24) then
         ndtday=ndh/24
         ndh=MOD(ndh,24)
      endif

c --- Apply change in hours
      ihour=ihour+ndh
      if(ihour.LT.0) then
c ---    Move days back 1 day and hours forward 24
         ndtday=ndtday-1
         ihour=ihour+24
      elseif(ihour.GE.24) then
c ---    Move days forward 1 day and hours back 24
         ndtday=ndtday+1
         ihour=ihour-24
      endif

c --- Apply change in days
      julday=julday+ndtday

      if(julday.GT.0) then
c ---    Current year or beyond
10       lastday=365
         if(MOD(iyear,4).EQ.0) lastday=366
         if(julday.GT.lastday) then
c ---       Move years forward 1 year and days back
            julday=julday-lastday
            iyear=iyear+1
         else
            return
         endif
         goto 10
      else
c ---    Previous year or beyond
c ---    Move years back and days forward
20       iyear=iyear-1
         lastday=365
         if(MOD(iyear,4).EQ.0) lastday=366
         julday=julday+lastday
         if(julday.GT.0) return
         goto 20
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine time_diff(iolst,cfrom,i8time1,i8time2,
     &                     ndelday,ndelhour,ndelsecond)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140219        TIME_DIFF
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Compute time interval (i8time2-i8time1)
c
c --- UPDATES:
c --- v1.22 (131218) to v2.0 (140219)
c        1. Remove IERR argument and add new string CFROM to arguments
c           of calls
c --- v1.21 (130605) to v1.22 (131218)
c        1. Revert to implicit INT_8 to INT_4 conversion 
c --- v1.0 (120518) to v1.21 (130605)
c        1. Revise integer names to start with: i,j,k,l,m,n
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c             I8TIME1 - integer*8 - Period start (YYYYJJJHHSSSS)
c             I8TIME2 - integer*8 - Period end   (YYYYJJJHHSSSS)
c
c --- OUTPUT:
c             ndelday - integer   - Time difference (whole days)
c            ndelhour - integer   - Time difference (whole hours)
c          ndelsecond - integer   - Time difference (whole seconds)
c
c --- TIME_DIFF called by:  ()
c --- TIME_DIFF calls:      DATE_HRSEC
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: cfrom
      integer(kind=8)  :: i8time1,i8time2
      integer          :: iolst,ndelday,ndelhour,ndelsecond,ierr

c --- Locals
      logical          :: lswap
      integer(kind=8)  :: i8diff
      integer          :: iyear1,jul1,ihour1,isec1
      integer          :: iyear2,jul2,ihour2,isec2
      integer          :: i

c --- Initializations
      ierr=0
      ndelday=0
      ndelhour=0
      ndelsecond=0

c --- Simple difference of times
      i8diff=i8time2-i8time1

      if(ABS(i8diff).LE.3600_8) then
c ---    Less than 1 hour difference
         ndelsecond=i8diff
         return
      endif

c --- Work with positive time differences
      lswap=.FALSE.
      if(i8diff.LT.0_8) then
         lswap=.TRUE.
      endif

      if(lswap) then
         call DATE_HRSEC(iolst,'TIME_DIFF:'//cfrom,'TOINDV',
     &                   iyear1,jul1,ihour1,isec1,i8time2)
         call DATE_HRSEC(iolst,'TIME_DIFF:'//cfrom,'TOINDV',
     &                   iyear2,jul2,ihour2,isec2,i8time1)
      else
         call DATE_HRSEC(iolst,'TIME_DIFF:'//cfrom,'TOINDV',
     &                   iyear1,jul1,ihour1,isec1,i8time1)
         call DATE_HRSEC(iolst,'TIME_DIFF:'//cfrom,'TOINDV',
     &                   iyear2,jul2,ihour2,isec2,i8time2)
      endif

c --- Difference in whole days
      if(iyear2.NE.iyear1) then
c ---    Step through each year summing the days
         do i=iyear1,iyear2
            if(i.EQ.iyear1) then
               ndelday=365-jul1
               if(MOD(iyear1,4).EQ.0) ndelday=ndelday+1
            elseif(i.EQ.iyear2) then
               ndelday=ndelday+jul2
            else
               ndelday=ndelday+365
               if(MOD(i,4).EQ.0) ndelday=ndelday+1
            endif
         enddo
      else
         ndelday=jul2-jul1
      endif

c --- Difference in whole hours
      ndelhour=ihour2-ihour1
      if(ndelhour.LT.0) then
c ---    Add 24 hours and remove 1 day
         ndelhour=ndelhour+24
         ndelday=ndelday-1
      endif

c --- Difference in whole seconds
      ndelsecond=isec2-isec1
      if(ndelsecond.LT.0) then
c ---    Add 3600 seconds and remove 1 hour
         ndelsecond=ndelsecond+3600
         ndelhour=ndelhour-1
      endif

      if(lswap) then
         ndelday=-ndelday
         ndelhour=-ndelhour
         ndelsecond=-ndelsecond
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine cond_hr_day(iolst,cfrom,action,i8datim)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140219      COND_HR_DAY
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Condition date-time at hour and day boundaries
c
c --- UPDATES:
c --- v1.22 (131218) to v2.0 (140219)
c        1. Remove IERR argument and add new string CFROM to arguments
c           of calls
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*     - Action expected: convert date-time
c                                   as needed to conform to ---
c                                   HR00S0000
c                                     Midnight is 00:00:00
c                                     All hours are HH, 0000 seconds
c                                   HR24S0000
c                                     Midnight is 24:00:00
c                                     All hours are HH, 0000 seconds
c                                   HR00S3600
c                                     Midnight is 23:59:60
c                                     All hours are HH-1, 3600 seconds
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c             I8DATIM - integer*8 - Date-Time (YYYYJJJHHSSSS)
c
c --- COND_HR_DAY called by:  ()
c --- COND_HR_DAY calls:      DATE_HRSEC, TIME_INCR
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action, cfrom
      integer          :: iolst
      integer*8        :: i8datim

c --- Locals
      integer          :: iyear,julday,ihour,isecond
      integer          :: ndthr,ndtsec

c --- Split out date-time components
      call DATE_HRSEC(iolst,'COND_HR_DAY:'//cfrom,'TOINDV',
     &                iyear,julday,ihour,isecond,i8datim)

c --- Seconds must be 0 or 3600 for an hour (or day) boundary
      if(isecond.EQ.0 .OR. isecond.EQ.3600) then
         SELECT CASE (action)

         CASE ("HR00S0000")
c ---       All times on the hour have zero seconds
            if(isecond.EQ.3600) then
               ihour=ihour+1
               isecond=0
            endif
c ---       Midnight is 00:00:00
            if(ihour.GE.24) then
               ndtsec=0
               ndthr=ihour-23
               ihour=23
               call TIME_INCR(iolst,'COND_HR_DAY:'//cfrom,
     &                        iyear,julday,ihour,isecond,ndthr,ndtsec)
            endif

         CASE ("HR24S0000")
c ---       All times on the hour have zero seconds
            if(isecond.EQ.3600) then
               ihour=ihour+1
               isecond=0
            endif
c ---       Midnight is 24:00:00
            if(ihour.EQ.0) then
               ndtsec=0
               ndthr=-1
               call TIME_INCR(iolst,'COND_HR_DAY:'//cfrom,
     &                        iyear,julday,ihour,isecond,ndthr,ndtsec)
               ihour=ihour+1
            endif

         CASE ("HR00S3600")
c ---       All times on the hour have 3600 seconds
            if(isecond.EQ.0) then
               ndtsec=0
               ndthr=-1
               call TIME_INCR(iolst,'COND_HR_DAY:'//cfrom,
     &                        iyear,julday,ihour,isecond,ndthr,ndtsec)
               isecond=3600
            endif

         CASE DEFAULT
c ---       Bad calling action
            write(iolst,*)
            write(iolst,*)'ERROR in COND_HR_DAY:  Unknown ACTION = ',
     &                     action
            write(iolst,*)'Expected: HR00S0000, HR24S0000, HR00S3600'
            write(iolst,*)'Called from  '//cfrom
            stop 'HALT in COND_HR_DAY -- see list file'

         END SELECT
      endif

c --- Combine date-time components
      call DATE_HRSEC(iolst,'COND_HR_DAY:'//cfrom,'TOSNGL',
     &                iyear,julday,ihour,isecond,i8datim)

      return
      end

c----------------------------------------------------------------------
      subroutine timechk(iolst,action,i8beg_a,i8end_a,i8beg_b,i8end_b,
     &                   iresult)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 2.2.0          Level: 140219          TIMECHK
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Checks start and end-times of 2 periods and either
c               1: characterizes the overlap of the first period (A)
c                  with the second (B), or
c               2: checks that length of period B is an integer
c                  multiple of the length of period A
c
c --- INPUTS:
c           IOLST - integer   - Unit number of output list file
c          ACTION - char*     - Action to perform
c                                 OVERLAP checks period overlap
c                                 LENGTH  checks period lengths
c         i8beg_a - integer*8 - Start date-time of period A
c         i8end_a - integer*8 - End date-time of period A
c         i8beg_b - integer*8 - Start date-time of period B
c         i8end_b - integer*8 - End date-time of period B
c
c --- OUTPUT:
c
c         iresult - integer   - OVERLAP assessment
c                                    If A is a definite period
c                               -2:  BOTH ends of A are BEFORE B
c                               -1:  A starts BEFORE B and ends IN B
c                                0:  A starts & ends IN B
c                                1:  A starts IN B and ends AFTER B
c                                2:  BOTH ends of A are AFTER B
c                               99:  A starts BEFORE B and ends AFTER B
c                                    If A is a point
c                               -2:  Pt A is AT or BEFORE start of B
c                                0:  Pt A is IN or AT the end of B
c                                2:  Pt A is AFTER the end of B
c         iresult - integer   - LENGTH assessment
c                                0:  B is NOT an integer multiple of A
c                                m:  multiple
c
c --- TIMECHK called by:  ()
c --- TIMECHK calls:      TIME_DIFF
c----------------------------------------------------------------------

      implicit none

c --- Arguments
      character(len=*) :: action
      integer          :: iolst, iresult
      integer(kind=8)  :: i8beg_a,i8end_a, i8beg_b,i8end_b

c --- Locals
      integer          :: nd,nh,ns
      integer          :: nsec_a,nsec_b

      SELECT CASE (action)

      CASE ("OVERLAP")

c ---    Special case of Pd A with zero length (point)
         if(i8beg_a.EQ.i8end_a) then
            if(i8end_a.GT.i8end_b) then
c ---          Pt A is after end of Pd B
               iresult=2
            elseif(i8end_a.LE.i8beg_b) then
c ---          Pt A is before or at start Pd B
               iresult=-2
            else
c ---          Pt A is within Pd B
               iresult=0
            endif
            return
         endif

c ---    Regular case
         if(i8beg_a.GE.i8end_b) then
c ---       Start of Pd A lies after or at end of Pd B
            iresult=2
         elseif(i8beg_a.GE.i8beg_b) then
c ---       Start of Pd A lies in Pd B
            if(i8end_a.LE.i8end_b) then
c ---          Pd A starts & ends in Pd B
               iresult=0
            else
c ---          Pd A starts in B and ends after B
               iresult=1
            endif
         else
c ---       Pd A starts before Pd B starts
            if(i8end_a.LE.i8beg_b) then
c ---          End of Pd A lies before or at start of Pd B
               iresult=-2
            elseif(i8end_a.LE.i8beg_b) then
c ---          End of Pd A lies in Pd B
               iresult=-1
            else
c ---          Pd A starts before Pd B and ends after Pd B
               iresult=99
            endif
         endif

      CASE ("LENGTH")
c ---    Get length of each period (seconds)
         call TIME_DIFF(iolst,'TIMECHK',i8beg_a,i8end_a,nd,nh,ns)
         nsec_a=((nd*24)+nh)*3600+ns
         call TIME_DIFF(iolst,'TIMECHK',i8beg_b,i8end_b,nd,nh,ns)
         nsec_b=((nd*24)+nh)*3600+ns
         if(nsec_a.EQ.nsec_b) then
c ---       Periods are the same
            iresult=1
         elseif(nsec_a.GT.nsec_b) then
c ---       Period A exceeds period B
            iresult=0
         elseif(MOD(nsec_b,nsec_a).EQ.0) then
c ---       OK
            iresult=nsec_b/nsec_a
         else
c ---       B is NOT a multiple of A
            iresult=0
         endif

      CASE DEFAULT
c ---    Bad calling action
         write(iolst,*)
         write(iolst,*)'ERROR in TIMECHK:  Unknown ACTION = ',action
         write(iolst,*)'Expected: OVERLAP or LENGTH'
         stop 'HALTED in TIMECHK -- see list file'

      END SELECT

      return
      end

c----------------------------------------------------------------------
      subroutine get_rll(iolst,cfrom,action,cll_in,rll_out)
c----------------------------------------------------------------------
c --- UTILITY   Version: 2.2.0          Level: 140219          GET_RLL
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Pull real latitude or longitude from a character string
c               (Current logic may not work for exponential notation)
c
c --- UPDATES:
c --- v1.23 (140106) to v2.0 (140219)
c        1. Remove IERR argument and issue STOP within subroutine, and
c           add new string CFROM to argument to report calling-path
c           to list file.
c --- v1.2 (130419) to v1.23 (140106)
c        1. Demand inputs be in the following ranges:
c              0E to 180E
c              0W to 180W
c              0N to 90N
c              0S to 90S
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c               CFROM - char*     - Called-From string to report error
c              ACTION - char*     - Action expected
c                                   LAT expects N or S in string
c                                   LON expects E or W in string
c              CLL_IN - char*     - Input character string
c
c --- OUTPUT:
c             RLL_OUT - real      - Output real is either:
c                                   Lat (Northing) -90. to 90. degrees
c                                   Lon (Easting) -180. to 180. degrees
c                IERR - integer   - Error flag (0 = No Error)
c
c --- GET_RLL called by:  ()
c --- GET_RLL calls:      MK_CASE, RM_BLANK
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action, cll_in, cfrom
      integer          :: iolst
      real             :: rll_out

c --- Locals
      character(len=48) :: cll_temp
      integer i,k
      integer iproc
c ---    N if iproc=1
c ---    S if iproc=2
c ---    E if iproc=3
c ---    W if iproc=4


c --- Initializations
      iproc=0

c --- Place input string into temp variable
      cll_temp='                                                '
      k=LEN_TRIM(cll_in)
      cll_temp(1:k)=cll_in

c --- Convert string to upper case
      call MK_CASE(iolst,'GET_RLL'//cfrom,'UPPER',cll_temp)

c --- Check string for expected N,S,E,W
      if(action.EQ.'LAT') then
         i=INDEX(cll_temp,'N')
         if(i.GT.0) then
            iproc=1
c ---       Remove character
            cll_temp(i:i)=' '
         else
            i=INDEX(cll_temp,'S')
            if(i.GT.0) then
               iproc=2
c ---          Remove character
               cll_temp(i:i)=' '
            endif
         endif
      elseif(action.EQ.'LON') then
         i=INDEX(cll_temp,'E')
         if(i.GT.0) then
            iproc=3
c ---       Remove character
            cll_temp(i:i)=' '
         else
            i=INDEX(cll_temp,'W')
            if(i.GT.0) then
               iproc=4
c ---          Remove character
               cll_temp(i:i)=' '
            endif
         endif
      else
         write(iolst,*)'ERROR in GET_RLL -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: LAT or LON'
         write(iolst,*)'   ACTION found   : ',action
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in GET_RLL -- see list file'
      endif
      if(iproc.EQ.0) then
         write(iolst,*)'ERROR in GET_RLL -- Invalid LAT/LON'
         write(iolst,*)'   Expected N or S for LAT'
         write(iolst,*)'   Expected E or W for LON'
         write(iolst,*)'   Found   : ',cll_in
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in GET_RLL -- see list file'
      endif

c --- Remove all blanks (should leave just number at this point)
      call RM_BLANK(cll_temp)
      k=LEN_TRIM(cll_temp)

c --- Condition decimal point
      i=INDEX(cll_temp,'.')
      if(i.EQ.0) then
         k=k+1
         cll_temp(k:k)='.'
      endif
      
c --- Allow for up to 20 characters in final real
      if(k.GT.20) then
         write(iolst,*)'PROBLEM in GET_RLL -- real # is too long'
         write(iolst,*)'   Code limits real number to 20 characters'
         write(iolst,*)'   Number found: ',cll_temp(1:k)
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in GET_RLL -- see list file'
      endif

c --- Read string into real number
      read(cll_temp(1:20),'(f20.0)') rll_out

c --- Demand real number >=0.0
      if(rll_out.LT.0.0) then
         write(iolst,*)'ERROR in GET_RLL -- Invalid LAT/LON'
         write(iolst,*)'   Expected POSITIVE N or S for LAT'
         write(iolst,*)'   Expected POSITIVE E or W for LON'
         write(iolst,*)'   Found   : ',cll_in
         write(iolst,*)'Called from  '//cfrom
         stop 'HALT in GET_RLL -- see list file'
      endif

c --- Flip sign when needed
      if(iproc.EQ.2 .OR. iproc.EQ.4) then
c ---    S latitude to northing latitude, or
c ---    W longitude to easting longitude
         rll_out=-rll_out
      endif

c --- Condition longitude to +/-180 (should not be needed)
      if(iproc.GT.2) then
         if(rll_out.LT.-180.) rll_out=rll_out+360.0
         if(rll_out.GT. 180.) rll_out=rll_out-360.0
      endif

      return
      end
